@@ -32,6 +32,7 @@
 #include <log4cplus/logger.h>
 #include <log4cplus/ndc.h>
 #include <log4cplus/mdc.h>
+#include <log4cplus/helpers/eventcounter.h>
 #include <log4cplus/helpers/loglog.h>
 #include <log4cplus/internal/customloglevelmanager.h>
 #include <log4cplus/internal/internal.h>
@@ -46,6 +47,7 @@
 #include <cstdio>
 #include <iostream>
 #include <stdexcept>
+#include <chrono>
 
 
 // Forward Declarations
@@ -146,6 +148,29 @@ instantiate_thread_pool ()
 #endif
 
 
+//! Helper structure for holding progschj::ThreadPool pointer.
+//! It is necessary to have this so that we can correctly order
+//! destructors between Hierarchy and the progschj::ThreadPool.
+//! Hierarchy wants to wait for outstading logging to finish
+//! therefore the ThreadPool can only be destroyed after that.
+struct ThreadPoolHolder
+{
+#if ! defined (LOG4CPLUS_SINGLE_THREADED)
+    std::atomic<progschj::ThreadPool*> thread_pool{};
+#endif
+
+    ThreadPoolHolder () = default;
+    ThreadPoolHolder (ThreadPoolHolder const&) = delete;
+    ~ThreadPoolHolder ()
+    {
+#if ! defined (LOG4CPLUS_SINGLE_THREADED)
+        auto const tp = thread_pool.exchange(nullptr, std::memory_order_release);
+        delete tp;
+#endif
+    }
+};
+
+
 //! Default context.
 struct DefaultContext
 {
@@ -160,10 +185,35 @@ struct DefaultContext
     spi::LayoutFactoryRegistry layout_factory_registry;
     spi::FilterFactoryRegistry filter_factory_registry;
     spi::LocaleFactoryRegistry locale_factory_registry;
-#if ! defined (LOG4CPLUS_SINGLE_THREADED)
-    std::unique_ptr<progschj::ThreadPool> thread_pool {instantiate_thread_pool ()};
-#endif
     Hierarchy hierarchy;
+    ThreadPoolHolder thread_pool;
+    std::atomic<bool> block_on_full {true};
+
+#if ! defined (LOG4CPLUS_SINGLE_THREADED)
+    progschj::ThreadPool *
+    get_thread_pool (bool init)
+    {
+        if (init) {
+            std::call_once (thread_pool_once, [&] {
+                thread_pool.thread_pool.store (instantiate_thread_pool ().release (), std::memory_order_release);
+            });
+        }
+        // cppreference.com says: The specification of release-consume ordering
+        // is being revised, and the use of memory_order_consume is temporarily
+        // discouraged. Thus, let's use memory_order_acquire.
+        return thread_pool.thread_pool.load (std::memory_order_acquire);
+    }
+
+    void
+    shutdown_thread_pool ()
+    {
+        auto const tp = thread_pool.thread_pool.exchange(nullptr, std::memory_order_release);
+        delete tp;
+    }
+
+private:
+    std::once_flag thread_pool_once;
+#endif
 };
 
 
@@ -320,11 +370,38 @@ void
 enqueueAsyncDoAppend (SharedAppenderPtr const & appender,
     spi::InternalLoggingEvent const & event)
 {
-    get_dc ()->thread_pool->enqueue (
-        [=] ()
+    static helpers::SteadyClockGate gate (helpers::SteadyClockGate::Duration {std::chrono::minutes (5)});
+
+    DefaultContext * dc = get_dc ();
+    progschj::ThreadPool * tp = dc->get_thread_pool (true);
+    if (dc->block_on_full)
+        tp->enqueue_block ([=] () { appender->asyncDoAppend (event); });
+    else
+    {
+        std::future<void> future = tp->enqueue ([=] () { appender->asyncDoAppend (event); });
+        if (future.wait_for (std::chrono::seconds (0)) == std::future_status::ready)
         {
-            appender->asyncDoAppend (event);
-        });
+            try
+            {
+                future.get ();
+            }
+            catch (const progschj::would_block &)
+            {
+                gate.record_event ();
+                helpers::SteadyClockGate::Info info;
+                if (gate.latch_open (info))
+                {
+                    helpers::LogLog & loglog = helpers::getLogLog ();
+                    log4cplus::tostringstream oss;
+                    oss << LOG4CPLUS_TEXT ("Asynchronous logging queue is full. Dropped ")
+                        << info.count << LOG4CPLUS_TEXT (" events in last ")
+                        << std::chrono::duration_cast<std::chrono::seconds> (info.time_span).count ()
+                        << LOG4CPLUS_TEXT (" seconds");
+                    loglog.warn (oss.str ());
+                }
+            }
+        }
+    }
 }
 
 #endif
@@ -334,9 +411,9 @@ shutdownThreadPool ()
 {
 #if ! defined (LOG4CPLUS_SINGLE_THREADED)
     DefaultContext * const dc = get_dc (false);
-    if (dc && dc->thread_pool)
+    if (dc)
     {
-        dc->thread_pool.reset ();
+        dc->shutdown_thread_pool ();
     }
 #endif
 }
@@ -347,10 +424,11 @@ waitUntilEmptyThreadPoolQueue ()
 {
 #if ! defined (LOG4CPLUS_SINGLE_THREADED)
     DefaultContext * const dc = get_dc (false);
-    if (dc && dc->thread_pool)
+    progschj::ThreadPool * tp;
+    if (dc && (tp = dc->get_thread_pool (false)))
     {
-        dc->thread_pool->wait_until_empty ();
-        dc->thread_pool->wait_until_nothing_in_flight ();
+        tp->wait_until_empty ();
+        tp->wait_until_nothing_in_flight ();
     }
 #endif
 }
@@ -557,8 +635,8 @@ initialize ()
 void
 deinitialize ()
 {
-    shutdownThreadPool();
     Logger::shutdown ();
+    shutdownThreadPool();
 }
 
 
@@ -573,7 +651,7 @@ threadCleanup ()
     //
     // This function can be called from TLS initializer/terminator by loader
     // when log4cplus is compiled and linked to as a static library. In case of
-    // other threads temination, it should do its job and free per-thread
+    // other threads termination, it should do its job and free per-thread
     // data. However, when the whole process is being terminated, it is called
     // after the CRT has been uninitialized and the CRT heap is not available
     // any more. In such case, instead of crashing, we just give up and leak
@@ -605,7 +683,7 @@ void
 setThreadPoolSize (std::size_t LOG4CPLUS_THREADED (pool_size))
 {
 #if ! defined (LOG4CPLUS_SINGLE_THREADED)
-    std::unique_ptr<progschj::ThreadPool> & thread_pool = get_dc ()->thread_pool;
+    auto const thread_pool = get_dc ()->get_thread_pool (true);
     if (thread_pool)
         thread_pool->set_pool_size (pool_size);
 
@@ -613,6 +691,24 @@ setThreadPoolSize (std::size_t LOG4CPLUS_THREADED (pool_size))
 }
 
 
+void
+setThreadPoolQueueSizeLimit (std::size_t LOG4CPLUS_THREADED (queue_size_limit))
+{
+#if ! defined (LOG4CPLUS_SINGLE_THREADED)
+    auto const thread_pool = get_dc ()->get_thread_pool (true);
+    if (thread_pool)
+        thread_pool->set_queue_size_limit (queue_size_limit);
+
+#endif
+}
+
+
+void
+setThreadPoolBlockOnFull (bool block)
+{
+    get_dc ()->block_on_full.store (block);
+}
+
 static
 void
 freeTLSSlot ()
