@@ -34,6 +34,7 @@
 #include <locale>
 #include <fstream>
 #include <sstream>
+#include <iostream>
 #include <log4cplus/streams.h>
 #include <log4cplus/fstreams.h>
 #include <log4cplus/helpers/stringhelper.h>
@@ -41,6 +42,8 @@
 #include <log4cplus/internal/internal.h>
 #include <log4cplus/internal/env.h>
 #include <log4cplus/helpers/loglog.h>
+#include <log4cplus/exception.h>
+#include <log4cplus/configurator.h>
 
 #if defined (LOG4CPLUS_WITH_UNIT_TESTS)
 #include <catch.hpp>
@@ -162,7 +165,7 @@ imbue_file_from_flags (tistream & file, unsigned flags)
                 // TODO: This should actually be a custom "null" facet
                 // that just copies the chars to wchar_t buffer.
                 new std::codecvt<wchar_t, char, std::mbstate_t>));
-    break;
+        break;
 
 #endif
 
@@ -186,6 +189,111 @@ imbue_file_from_flags (tistream & file, unsigned flags)
 } // namespace
 
 
+/**
+ * Perform variable substitution in string <code>val</code> from
+ * environment variables.
+ *
+ * <p>The variable substitution delimeters are <b>${</b> and <b>}</b>.
+ *
+ * <p>For example, if the System properties contains "key=value", then
+ * the call
+ * <pre>
+ * string s;
+ * substEnvironVars(s, "Value of key is ${key}.");
+ * </pre>
+ *
+ * will set the variable <code>s</code> to "Value of key is value.".
+ *
+ * <p>If no value could be found for the specified key, then
+ * substitution defaults to the empty string.
+ *
+ * <p>For example, if there is no environment variable "inexistentKey",
+ * then the call
+ *
+ * <pre>
+ * string s;
+ * substEnvironVars(s, "Value of inexistentKey is [${inexistentKey}]");
+ * </pre>
+ * will set <code>s</code> to "Value of inexistentKey is []"
+ *
+ * @param val The string on which variable substitution is performed.
+ * @param dest The result.
+ */
+bool
+substVars (tstring & dest, const tstring & val,
+    helpers::Properties const & props, helpers::LogLog& loglog,
+    unsigned flags)
+{
+    static tchar const DELIM_START[] = LOG4CPLUS_TEXT("${");
+    static tchar const DELIM_STOP[] = LOG4CPLUS_TEXT("}");
+    static std::size_t const DELIM_START_LEN = 2;
+    static std::size_t const DELIM_STOP_LEN = 1;
+
+    tstring::size_type i = 0;
+    tstring::size_type var_start, var_end;
+    tstring pattern (val);
+    tstring key;
+    tstring replacement;
+    bool changed = false;
+    bool const empty_vars
+        = !! (flags & PropertyConfigurator::fAllowEmptyVars);
+    bool const shadow_env
+        = !! (flags & PropertyConfigurator::fShadowEnvironment);
+    bool const rec_exp
+        = !! (flags & PropertyConfigurator::fRecursiveExpansion);
+
+    while (true)
+    {
+        // Find opening paren of variable substitution.
+        var_start = pattern.find(DELIM_START, i);
+        if (var_start == tstring::npos)
+        {
+            dest = pattern;
+            return changed;
+        }
+
+        // Find closing paren of variable substitution.
+        var_end = pattern.find(DELIM_STOP, var_start);
+        if (var_end == tstring::npos)
+        {
+            tostringstream buffer;
+            buffer << '"' << pattern
+                    << "\" has no closing brace. "
+                    << "Opening brace at position " << var_start << ".";
+            loglog.error(buffer.str());
+            dest = val;
+            return false;
+        }
+
+        key.assign (pattern, var_start + DELIM_START_LEN,
+            var_end - (var_start + DELIM_START_LEN));
+        replacement.clear ();
+        if (shadow_env)
+            replacement = props.getProperty (key);
+        if (! shadow_env || (! empty_vars && replacement.empty ()))
+            internal::get_env_var (replacement, key);
+
+        if (empty_vars || ! replacement.empty ())
+        {
+            // Substitute the variable with its value in place.
+            pattern.replace (var_start, var_end - var_start + DELIM_STOP_LEN,
+                replacement);
+            changed = true;
+            if (rec_exp)
+                // Retry expansion on the same spot.
+                continue;
+            else
+                // Move beyond the just substituted part.
+                i = var_start + replacement.size ();
+        }
+        else
+            // Nothing has been subtituted, just move beyond the
+            // unexpanded variable.
+            i = var_end + DELIM_STOP_LEN;
+    } // end while loop
+
+} // end substVars()
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // Properties ctors and dtor
@@ -219,7 +327,7 @@ Properties::Properties(const tstring& inputFile, unsigned f)
         std::ios::binary);
     if (! file.good ())
         helpers::getLogLog ().error (LOG4CPLUS_TEXT ("could not open file ")
-            + inputFile);
+            + inputFile, (flags & Properties::fThrow) != 0);
 
     init(file);
 }
@@ -254,14 +362,17 @@ Properties::init(tistream& input)
             tstring included (buffer, 8) ;
             trim_ws (included);
 
+            tstring subIncluded;
+            helpers::substVars(subIncluded, included, *this, helpers::getLogLog(), 0);
+
             tifstream file;
             imbue_file_from_flags (file, flags);
 
-            file.open (LOG4CPLUS_FSTREAM_PREFERED_FILE_NAME(included).c_str(),
+            file.open (LOG4CPLUS_FSTREAM_PREFERED_FILE_NAME(subIncluded).c_str(),
                 std::ios::binary);
             if (! file.good ())
                 helpers::getLogLog ().error (
-                    LOG4CPLUS_TEXT ("could not open file ") + included);
+                    LOG4CPLUS_TEXT ("could not open file ") + subIncluded);
 
             init (file);
         }
@@ -537,6 +648,14 @@ CATCH_TEST_CASE ("Properties", "[properties]")
         CATCH_REQUIRE (std::find (std::begin (names), std::end (names),
                 PROP_SECOND) != std::end (names));
     }
+
+    CATCH_SECTION ("throw on nonexistent file")
+    {
+        auto && f = [] {
+            Properties props (LOG4CPLUS_TEXT ("xxx does not exist"), Properties::fThrow);
+        };
+        CATCH_REQUIRE_THROWS_AS (f (), log4cplus::exception);
+    }
 }
 #endif
 
