@@ -139,7 +139,7 @@ main(int argc, char * argv[])
     }
     catch(...) {
         tcout << LOG4CPLUS_TEXT("Exception...") << endl;
-        LOG4CPLUS_FATAL(root, "Exception occured...");
+        LOG4CPLUS_FATAL(root, "Exception occurred...");
     }
 
     tcout << LOG4CPLUS_TEXT("Exiting main()...") << endl;
