@@ -56,7 +56,7 @@ main(int argc, char * argv[])
     }
     catch(...) {
         tcout << LOG4CPLUS_TEXT("Exception...") << endl;
-        LOG4CPLUS_FATAL(root, LOG4CPLUS_TEXT("Exception occured..."));
+        LOG4CPLUS_FATAL(root, LOG4CPLUS_TEXT("Exception occurred..."));
     }
 
     tcout << LOG4CPLUS_TEXT("Exiting main()...") << endl;
